@@ -1,18 +1,15 @@
-//! [`Output`][Output] assertions.
-//!
-//! [Output]: https://doc.rust-lang.org/std/process/struct.Output.html
+//! [`std::process::Output`] assertions.
 
+use std::borrow::Cow;
 use std::fmt;
 use std::process;
 use std::str;
 
-use predicates;
 use predicates::str::PredicateStrExt;
-use predicates_core;
 use predicates_tree::CaseTreeExt;
 
-use crate::output::dump_buffer;
 use crate::output::output_fmt;
+use crate::output::DebugBytes;
 
 /// Assert the state of an [`Output`].
 ///
@@ -29,7 +26,7 @@ use crate::output::output_fmt;
 ///     .success();
 /// ```
 ///
-/// [`Output`]: https://doc.rust-lang.org/std/process/struct.Output.html
+/// [`Output`]: std::process::Output
 pub trait OutputAssertExt {
     /// Wrap with an interface for that provides assertions on the [`Output`].
     ///
@@ -46,7 +43,7 @@ pub trait OutputAssertExt {
     ///     .success();
     /// ```
     ///
-    /// [`Output`]: https://doc.rust-lang.org/std/process/struct.Output.html
+    /// [`Output`]: std::process::Output
     fn assert(self) -> Assert;
 }
 
@@ -58,7 +55,12 @@ impl OutputAssertExt for process::Output {
 
 impl<'c> OutputAssertExt for &'c mut process::Command {
     fn assert(self) -> Assert {
-        let output = self.output().unwrap();
+        let output = match self.output() {
+            Ok(output) => output,
+            Err(err) => {
+                panic!("Failed to spawn {:?}: {}", self, err);
+            }
+        };
         Assert::new(output).append_context("command", format!("{:?}", self))
     }
 }
@@ -80,8 +82,7 @@ impl<'c> OutputAssertExt for &'c mut process::Command {
 ///     .success();
 /// ```
 ///
-/// [`Output`]: https://doc.rust-lang.org/std/process/struct.Output.html
-/// [`OutputAssertExt`]: trait.OutputAssertExt.html
+/// [`Output`]: std::process::Output
 pub struct Assert {
     output: process::Output,
     context: Vec<(&'static str, Box<dyn fmt::Display>)>,
@@ -90,7 +91,7 @@ pub struct Assert {
 impl Assert {
     /// Create an `Assert` for a given [`Output`].
     ///
-    /// [`Output`]: https://doc.rust-lang.org/std/process/struct.Output.html
+    /// [`Output`]: std::process::Output
     pub fn new(output: process::Output) -> Self {
         Self {
             output,
@@ -123,7 +124,7 @@ impl Assert {
 
     /// Access the contained [`Output`].
     ///
-    /// [`Output`]: https://doc.rust-lang.org/std/process/struct.Output.html
+    /// [`Output`]: std::process::Output
     pub fn get_output(&self) -> &process::Output {
         &self.output
     }
@@ -147,14 +148,14 @@ impl Assert {
             let actual_code = self.output.status.code().unwrap_or_else(|| {
                 panic!(
                     "Unexpected failure.\ncode=<interrupted>\nstderr=```{}```\n{}",
-                    dump_buffer(&self.output.stderr),
+                    DebugBytes::new(&self.output.stderr),
                     self
                 )
             });
             panic!(
                 "Unexpected failure.\ncode-{}\nstderr=```{}```\n{}",
                 actual_code,
-                dump_buffer(&self.output.stderr),
+                DebugBytes::new(&self.output.stderr),
                 self
             );
         }
@@ -201,9 +202,6 @@ impl Assert {
     ///
     /// Accepting a predicate:
     /// ```rust,no_run
-    /// extern crate assert_cmd;
-    /// extern crate predicates;
-    ///
     /// use assert_cmd::prelude::*;
     ///
     /// use std::process::Command;
@@ -242,8 +240,6 @@ impl Assert {
     ///     .code(&[2, 42] as &[i32]);
     /// ```
     ///
-    /// [`predicates`]: https://docs.rs/predicates
-    /// [`IntoCodePredicate`]: trait.IntoCodePredicate.html
     pub fn code<I, P>(self, pred: I) -> Self
     where
         I: IntoCodePredicate<P>,
@@ -274,9 +270,6 @@ impl Assert {
     ///
     /// Accepting a bytes predicate:
     /// ```rust,no_run
-    /// extern crate assert_cmd;
-    /// extern crate predicates;
-    ///
     /// use assert_cmd::prelude::*;
     ///
     /// use std::process::Command;
@@ -292,9 +285,6 @@ impl Assert {
     ///
     /// Accepting a `str` predicate:
     /// ```rust,no_run
-    /// extern crate assert_cmd;
-    /// extern crate predicates;
-    ///
     /// use assert_cmd::prelude::*;
     ///
     /// use std::process::Command;
@@ -336,8 +326,6 @@ impl Assert {
     ///     .stdout("hello\n");
     /// ```
     ///
-    /// [`predicates`]: https://docs.rs/predicates
-    /// [`IntoOutputPredicate`]: trait.IntoOutputPredicate.html
     pub fn stdout<I, P>(self, pred: I) -> Self
     where
         I: IntoOutputPredicate<P>,
@@ -366,9 +354,6 @@ impl Assert {
     ///
     /// Accepting a bytes predicate:
     /// ```rust,no_run
-    /// extern crate assert_cmd;
-    /// extern crate predicates;
-    ///
     /// use assert_cmd::prelude::*;
     ///
     /// use std::process::Command;
@@ -384,9 +369,6 @@ impl Assert {
     ///
     /// Accepting a `str` predicate:
     /// ```rust,no_run
-    /// extern crate assert_cmd;
-    /// extern crate predicates;
-    ///
     /// use assert_cmd::prelude::*;
     ///
     /// use std::process::Command;
@@ -428,8 +410,6 @@ impl Assert {
     ///     .stderr("world\n");
     /// ```
     ///
-    /// [`predicates`]: https://docs.rs/predicates
-    /// [`IntoOutputPredicate`]: trait.IntoOutputPredicate.html
     pub fn stderr<I, P>(self, pred: I) -> Self
     where
         I: IntoOutputPredicate<P>,
@@ -467,14 +447,11 @@ impl fmt::Debug for Assert {
 }
 
 /// Used by [`Assert::code`] to convert `Self` into the needed
-/// [`Predicate<i32>`].
+/// [`predicates_core::Predicate<i32>`].
 ///
 /// # Examples
 ///
 /// ```rust,no_run
-/// extern crate assert_cmd;
-/// extern crate predicates;
-///
 /// use assert_cmd::prelude::*;
 ///
 /// use std::process::Command;
@@ -493,9 +470,6 @@ impl fmt::Debug for Assert {
 ///     .assert()
 ///     .code(42);
 /// ```
-///
-/// [`Assert::code`]: struct.Assert.html#method.code
-/// [`Predicate<i32>`]: https://docs.rs/predicates-core/1.0.0/predicates_core/trait.Predicate.html
 pub trait IntoCodePredicate<P>
 where
     P: predicates_core::Predicate<i32>,
@@ -518,8 +492,8 @@ where
     }
 }
 
-// Keep `predicates` concrete Predicates out of our public API.
-/// [Predicate] used by [`IntoCodePredicate`] for code.
+/// Keep `predicates` concrete Predicates out of our public API.
+/// [predicates_core::Predicate] used by [`IntoCodePredicate`] for code.
 ///
 /// # Example
 ///
@@ -534,9 +508,6 @@ where
 ///     .assert()
 ///     .code(42);
 /// ```
-///
-/// [`IntoCodePredicate`]: trait.IntoCodePredicate.html
-/// [Predicate]: https://docs.rs/predicates-core/1.0.0/predicates_core/trait.Predicate.html
 #[derive(Debug)]
 pub struct EqCodePredicate(predicates::ord::EqPredicate<i32>);
 
@@ -590,8 +561,8 @@ impl IntoCodePredicate<EqCodePredicate> for i32 {
     }
 }
 
-// Keep `predicates` concrete Predicates out of our public API.
-/// [Predicate] used by [`IntoCodePredicate`] for iterables of codes.
+/// Keep `predicates` concrete Predicates out of our public API.
+/// [predicates_core::Predicate] used by [`IntoCodePredicate`] for iterables of codes.
 ///
 /// # Example
 ///
@@ -606,9 +577,6 @@ impl IntoCodePredicate<EqCodePredicate> for i32 {
 ///     .assert()
 ///     .code(&[2, 42] as &[i32]);
 /// ```
-///
-/// [`IntoCodePredicate`]: trait.IntoCodePredicate.html
-/// [Predicate]: https://docs.rs/predicates-core/1.0.0/predicates_core/trait.Predicate.html
 #[derive(Debug)]
 pub struct InCodePredicate(predicates::iter::InPredicate<i32>);
 
@@ -671,14 +639,11 @@ impl IntoCodePredicate<InCodePredicate> for &'static [i32] {
 }
 
 /// Used by [`Assert::stdout`] and [`Assert::stderr`] to convert Self
-/// into the needed [`Predicate<[u8]>`].
+/// into the needed [`predicates_core::Predicate<[u8]>`].
 ///
 /// # Examples
 ///
 /// ```rust,no_run
-/// extern crate assert_cmd;
-/// extern crate predicates;
-///
 /// use assert_cmd::prelude::*;
 ///
 /// use std::process::Command;
@@ -699,10 +664,6 @@ impl IntoCodePredicate<InCodePredicate> for &'static [i32] {
 ///     .assert()
 ///     .stdout("hello\n");
 /// ```
-///
-/// [`Assert::stdout`]: struct.Assert.html#method.stdout
-/// [`Assert::stderr`]: struct.Assert.html#method.stderr
-/// [`Predicate<[u8]>`]: https://docs.rs/predicates-core/1.0.0/predicates_core/trait.Predicate.html
 pub trait IntoOutputPredicate<P>
 where
     P: predicates_core::Predicate<[u8]>,
@@ -725,8 +686,8 @@ where
     }
 }
 
-// Keep `predicates` concrete Predicates out of our public API.
-/// [Predicate] used by [`IntoOutputPredicate`] for bytes.
+/// Keep `predicates` concrete Predicates out of our public API.
+/// [predicates_core::Predicate] used by [`IntoOutputPredicate`] for bytes.
 ///
 /// # Example
 ///
@@ -742,51 +703,51 @@ where
 ///     .assert()
 ///     .stderr(b"world\n" as &[u8]);
 /// ```
-///
-/// [`IntoOutputPredicate`]: trait.IntoOutputPredicate.html
-/// [Predicate]: https://docs.rs/predicates-core/1.0.0/predicates_core/trait.Predicate.html
 #[derive(Debug)]
-pub struct BytesContentOutputPredicate(predicates::ord::EqPredicate<&'static [u8]>);
+pub struct BytesContentOutputPredicate(Cow<'static, [u8]>);
 
 impl BytesContentOutputPredicate {
     pub(crate) fn new(value: &'static [u8]) -> Self {
-        let pred = predicates::ord::eq(value);
-        BytesContentOutputPredicate(pred)
+        BytesContentOutputPredicate(Cow::from(value))
+    }
+
+    pub(crate) fn from_vec(value: Vec<u8>) -> Self {
+        BytesContentOutputPredicate(Cow::from(value))
     }
 }
 
-impl predicates_core::reflection::PredicateReflection for BytesContentOutputPredicate {
-    fn parameters<'a>(
-        &'a self,
-    ) -> Box<dyn Iterator<Item = predicates_core::reflection::Parameter<'a>> + 'a> {
-        self.0.parameters()
-    }
-
-    /// Nested `Predicate`s of the current `Predicate`.
-    fn children<'a>(
-        &'a self,
-    ) -> Box<dyn Iterator<Item = predicates_core::reflection::Child<'a>> + 'a> {
-        self.0.children()
-    }
-}
+impl predicates_core::reflection::PredicateReflection for BytesContentOutputPredicate {}
 
 impl predicates_core::Predicate<[u8]> for BytesContentOutputPredicate {
     fn eval(&self, item: &[u8]) -> bool {
-        self.0.eval(item)
+        self.0.as_ref() == item
     }
 
-    fn find_case<'a>(
-        &'a self,
+    fn find_case(
+        &self,
         expected: bool,
         variable: &[u8],
-    ) -> Option<predicates_core::reflection::Case<'a>> {
-        self.0.find_case(expected, variable)
+    ) -> Option<predicates_core::reflection::Case> {
+        let actual = self.eval(variable);
+        if expected == actual {
+            Some(predicates_core::reflection::Case::new(Some(self), actual))
+        } else {
+            None
+        }
     }
 }
 
 impl fmt::Display for BytesContentOutputPredicate {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        self.0.fmt(f)
+        predicates::ord::eq(self.0.as_ref()).fmt(f)
+    }
+}
+
+impl IntoOutputPredicate<BytesContentOutputPredicate> for Vec<u8> {
+    type Predicate = BytesContentOutputPredicate;
+
+    fn into_output(self) -> Self::Predicate {
+        Self::Predicate::from_vec(self)
     }
 }
 
@@ -798,8 +759,8 @@ impl IntoOutputPredicate<BytesContentOutputPredicate> for &'static [u8] {
     }
 }
 
-// Keep `predicates` concrete Predicates out of our public API.
-/// [Predicate] used by [`IntoOutputPredicate`] for [`str`].
+/// Keep `predicates` concrete Predicates out of our public API.
+/// [predicates_core::Predicate] used by [`IntoOutputPredicate`] for [`str`].
 ///
 /// # Example
 ///
@@ -816,8 +777,6 @@ impl IntoOutputPredicate<BytesContentOutputPredicate> for &'static [u8] {
 ///     .stderr("world\n");
 /// ```
 ///
-/// [`IntoOutputPredicate`]: trait.IntoOutputPredicate.html
-/// [Predicate]: https://docs.rs/predicates-core/1.0.0/predicates_core/trait.Predicate.html
 /// [`str`]: https://doc.rust-lang.org/std/primitive.str.html
 #[derive(Debug, Clone)]
 pub struct StrContentOutputPredicate(
@@ -888,14 +847,12 @@ impl IntoOutputPredicate<StrContentOutputPredicate> for &'static str {
 }
 
 // Keep `predicates` concrete Predicates out of our public API.
-/// [Predicate] used by [`IntoOutputPredicate`] for [`Predicate<str>`].
+/// [predicates_core::Predicate] used by [`IntoOutputPredicate`] for
+/// [`Predicate<str>`][predicates_core::Predicate].
 ///
 /// # Example
 ///
 /// ```rust,no_run
-/// extern crate assert_cmd;
-/// extern crate predicates;
-///
 /// use assert_cmd::prelude::*;
 ///
 /// use std::process::Command;
@@ -908,9 +865,6 @@ impl IntoOutputPredicate<StrContentOutputPredicate> for &'static str {
 ///     .assert()
 ///     .stderr(predicate::str::similar("world\n"));
 /// ```
-///
-/// [`IntoOutputPredicate`]: trait.IntoOutputPredicate.html
-/// [Predicate]: https://docs.rs/predicates-core/1.0.0/predicates_core/trait.Predicate.html
 #[derive(Debug, Clone)]
 pub struct StrOutputPredicate<P: predicates_core::Predicate<str>>(
     predicates::str::Utf8Predicate<P>,
@@ -1043,6 +997,12 @@ mod test {
         assert!(pred.eval(b"Hello" as &[u8]));
     }
 
+    #[test]
+    fn into_output_from_vec() {
+        let pred = convert_output(vec![b'H', b'e', b'l', b'l', b'o']);
+        assert!(pred.eval(b"Hello" as &[u8]));
+    }
+
     #[test]
     fn into_output_from_str() {
         let pred = convert_output("Hello");
